// ReadSW_WriteLED.v

module ReadSW_WriteLED( clk, 
                        nreset,
                        bus_write_en, 
                        bus_read_en,
                        bus_addr,
                        bus_write_data, //data_in
                        sw_port,
                        bus_read_data, //data_out
                        led_port);
	
    input clk, nreset, bus_write_en, bus_read_en;
    input [7:0] bus_addr;
    input [1:0] sw_port; 
    input [31:0] bus_write_data;
    output [7:0] led_port;
    output reg [31:0] bus_read_data;
    
    reg  [2:0]   led_reg;
    reg  [1:0]   sw_reg;
    
    Decoder3to8 Decoder3to8_0 (.DataIn3bit(led_reg), .DataOut8bit(led_port));
	
    always@(posedge clk)
        if(~nreset) // Reset
            led_reg[2:0] <= 3'd0;
        else
            begin 
            case(bus_addr[3:2])
            2'b00:
                if(bus_write_en)
                    led_reg <= bus_write_data[2:0];
            2'b01:
                if(bus_read_en)
                    bus_read_data[31:0] <= {30'd0, sw_reg};
            default:
                if(bus_read_en)
                    bus_read_data[31:0] <= {29'd0, led_reg};
            endcase
            end
		
    always@*
        if(~nreset)
            sw_reg[1:0] <= 2'b00;
        else
            sw_reg[1:0] <= sw_port;
    
endmodule

// This module decodes the 2^3 or 8 possibilities of 3 input bits into 8 different/unique output bits.
module Decoder3to8( 
					DataIn3bit,
					DataOut8bit);
	
	input [2:0] DataIn3bit;
	output [7:0] DataOut8bit;
	
	reg [7:0] y;

	assign DataOut8bit = ~y; // LEDs are active low
	
	always @*
		begin case (DataIn3bit)
			3'b000: y = 8'b00000001; 
			3'b001: y = 8'b00000010; 
			3'b010: y = 8'b00000100; 
			3'b011: y = 8'b00001000; 
			3'b100: y = 8'b00010000; 
			3'b101: y = 8'b00100000; 
			3'b110: y = 8'b01000000; 
			3'b111: y = 8'b10000000; 
			default y = 0;
		endcase
	end
	
endmodule
